@@ -8,12 +8,15 @@ except ImportError:  # support for python2
 
 from opcua import ua
 from opcua.client.ua_client import UaClient
+from opcua.common.xmlimporter import XmlImporter
+from opcua.common.xmlexporter import XmlExporter
 from opcua.common.node import Node
 from opcua.common.manage_nodes import delete_nodes
 from opcua.common.subscription import Subscription
 from opcua.common import utils
 from opcua.crypto import security_policies
 from opcua.common.shortcuts import Shortcuts
+from opcua.common.structures_generator import StructGenerator
 use_crypto = True
 try:
     from opcua.crypto import uacrypto
@@ -92,6 +95,9 @@ class Client(object):
         """
         self.logger = logging.getLogger(__name__)
         self.server_url = urlparse(url)
+        #take initial username and password from the url
+        self._username = self.server_url.username
+        self._password = self.server_url.password
         self.name = "Pure Python Client"
         self.description = self.name
         self.application_uri = "urn:freeopcua:client"
@@ -125,9 +131,23 @@ class Client(object):
                     ep.SecurityMode == security_mode and
                     ep.SecurityPolicyUri == policy_uri):
                 return ep
-        raise ua.UaError("No matching endpoints: {}, {}".format(
+        raise ua.UaError("No matching endpoints: {0}, {1}".format(
                          security_mode, policy_uri))
 
+    def set_user(self, username):
+        """
+        Set user name for the connection.
+        initial user from the URL will be overwritten
+        """
+        self._username = username
+
+    def set_password(self, pwd):
+        """
+        Set user password for the connection.
+        initial password from the URL will be overwritten
+        """
+        self._password = pwd
+
     def set_security_string(self, string):
         """
         Set SecureConnection mode. String format:
@@ -142,7 +162,7 @@ class Client(object):
             return
         parts = string.split(',')
         if len(parts) < 4:
-            raise ua.UaError('Wrong format: `{}`, expected at least 4 comma-separated values'.format(string))
+            raise ua.UaError('Wrong format: `{0}`, expected at least 4 comma-separated values'.format(string))
         policy_class = getattr(security_policies, 'SecurityPolicy' + parts[0])
         mode = getattr(ua.MessageSecurityMode, parts[1])
         return self.set_security(policy_class, parts[2], parts[3],
@@ -224,7 +244,7 @@ class Client(object):
         self.send_hello()
         self.open_secure_channel()
         self.create_session()
-        self.activate_session(username=self.server_url.username, password=self.server_url.password, certificate=self.user_certificate)
+        self.activate_session(username=self._username, password=self._password, certificate=self.user_certificate)
 
     def disconnect(self):
         """
@@ -421,13 +441,13 @@ class Client(object):
             # see specs part 4, 7.36.3: if the token is NOT encrypted,
             # then the password only contains UTF-8 encoded password
             # and EncryptionAlgorithm is null
-            if self.server_url.password:
+            if self._password:
                 self.logger.warning("Sending plain-text password")
                 params.UserIdentityToken.Password = password
             params.UserIdentityToken.EncryptionAlgorithm = ''
-        elif self.server_url.password:
+        elif self._password:
             data, uri = self._encrypt_password(password, policy_uri)
-            params.UserIdentityToken.Password = data 
+            params.UserIdentityToken.Password = data
             params.UserIdentityToken.EncryptionAlgorithm = uri
         params.UserIdentityToken.PolicyId = self.server_policy_id(ua.UserTokenType.UserName, b"username_basic256")
 
@@ -438,7 +458,7 @@ class Client(object):
         passwd = password.encode("utf8")
         if self._server_nonce is not None:
             passwd += self._server_nonce
-        etoken = ua.pack_bytes(passwd)
+        etoken = ua.ua_binary.Primitives.Bytes.pack(passwd)
         data, uri = security_policies.encrypt_asymmetric(pubkey, etoken, policy_uri)
         return data, uri
 
@@ -471,16 +491,22 @@ class Client(object):
         returns a Subscription object which allow
         to subscribe to events or data on server
         handler argument is a class with data_change and/or event methods.
+        period argument is either a publishing interval in seconds or a
+        CreateSubscriptionParameters instance. The second option should be used,
+        if the opcua-server has problems with the default options.
         These methods will be called when notfication from server are received.
         See example-client.py.
         Do not do expensive/slow or network operation from these methods
         since they are called directly from receiving thread. This is a design choice,
         start another thread if you need to do such a thing.
         """
+
+        if isinstance(period, ua.CreateSubscriptionParameters):
+             return Subscription(self.uaclient, period, handler)
         params = ua.CreateSubscriptionParameters()
         params.RequestedPublishingInterval = period
-        params.RequestedLifetimeCount = 3000
-        params.RequestedMaxKeepAliveCount = 10000
+        params.RequestedLifetimeCount = 10000
+        params.RequestedMaxKeepAliveCount = 3000
         params.MaxNotificationsPerPublish = 10000
         params.PublishingEnabled = True
         params.Priority = 0
@@ -496,4 +522,74 @@ class Client(object):
 
     def delete_nodes(self, nodes, recursive=False):
         return delete_nodes(self.uaclient, nodes, recursive)
+
+    def import_xml(self, path):
+        """
+        Import nodes defined in xml
+        """
+        importer = XmlImporter(self)
+        return importer.import_xml(path)
+
+    def export_xml(self, nodes, path):
+        """
+        Export defined nodes to xml
+        """
+        exp = XmlExporter(self)
+        exp.build_etree(nodes)
+        return exp.write_xml(path)
+
+    def register_namespace(self, uri):
+        """
+        Register a new namespace. Nodes should in custom namespace, not 0.
+        This method is mainly implemented for symetry with server
+        """
+        ns_node = self.get_node(ua.NodeId(ua.ObjectIds.Server_NamespaceArray))
+        uries = ns_node.get_value()
+        if uri in uries:
+            return uries.index(uri)
+        uries.append(uri)
+        ns_node.set_value(uries)
+        return len(uries) - 1
+
+    def import_and_register_structures(self, nodes=None):
+        """
+        Download xml from given variable node defining custom structures.
+        If no no node is given, attemps to import variables from all nodes under
+        "0:OPC Binary"
+        the code is generated and imported on the fly. If you know the structures
+        are not going to be modified it might be interresting to copy the generated files
+        and include them in you code
+        """
+        if nodes is None:
+            nodes = []
+            for desc in self.nodes.opc_binary.get_children_descriptions():
+                if desc.BrowseName != ua.QualifiedName("Opc.Ua"):
+                    nodes.append(self.get_node(desc.NodeId))
+        self.logger.info("Importing structures from nodes: %s", nodes)
+        
+        structs_dict = {}
+        for node in nodes:
+            xml = node.get_value()
+            xml = xml.decode("utf-8")
+            name = "structures_" + node.get_browse_name().Name
+            gen = StructGenerator()
+            gen.make_model_from_string(xml)
+            gen.save_and_import(name + ".py", append_to=structs_dict)
+
+        # register classes
+        for desc in self.nodes.base_structure_type.get_children_descriptions():
+            # FIXME: maybe we should look recursively at children
+            # FIXME: we should get enoding and description but this is too 
+            # expensive. we take a shorcut and assume that browsename of struct 
+            # is the same as the name of the data type structure
+            if desc.BrowseName.Name in structs_dict:
+                struct_node = self.get_node(desc.NodeId)
+                refs = struct_node.get_references(ua.ObjectIds.HasEncoding, ua.BrowseDirection.Forward)
+                for ref in refs:
+                    if "Binary" in ref.BrowseName.Name:
+                        ua.register_extension_object(desc.BrowseName.Name, ref.NodeId, structs_dict[desc.BrowseName.Name])
+
+
+
+
             
