@@ -10,7 +10,7 @@ from functools import partial
 
 from opcua import ua
 from opcua.common import utils
-from opcua.common.uaerrors import UaError, BadTimeout, BadNoSubscription, BadSessionClosed
+from opcua.ua.uaerrors import UaError, BadTimeout, BadNoSubscription, BadSessionClosed
 
 
 class UASocketClient(object):
@@ -116,7 +116,7 @@ class UASocketClient(object):
         with self._lock:
             future = self._callbackmap.pop(request_id, None)
             if future is None:
-                raise ua.UaError("No future object found for request: {}, callbacks in list are {}".format(request_id, self._callbackmap.keys()))
+                raise ua.UaError("No future object found for request: {0}, callbacks in list are {1}".format(request_id, self._callbackmap.keys()))
         future.set_result(body)
 
     def _create_request_header(self, timeout=1000):
@@ -140,7 +140,7 @@ class UASocketClient(object):
     def disconnect_socket(self):
         self.logger.info("stop request")
         self._do_stop = True
-        self._socket.socket.shutdown(socket.SHUT_WR)
+        self._socket.socket.shutdown(socket.SHUT_RDWR)
         self._socket.socket.close()
 
     def send_hello(self, url):
@@ -159,7 +159,9 @@ class UASocketClient(object):
         request = ua.OpenSecureChannelRequest()
         request.Parameters = params
         future = self._send_request(request, message_type=ua.MessageType.SecureOpen)
-
+        
+        # FIXME: we have a race condition here
+        # we can get a packet with the new token id before we reach to store it..
         response = ua.OpenSecureChannelResponse.from_binary(future.result(self.timeout))
         response.ResponseHeader.ServiceResult.check()
         self._connection.set_channel(response.Parameters)
@@ -274,6 +276,16 @@ class UaClient(object):
         response.ResponseHeader.ServiceResult.check()
         return response.Results
 
+    def browse_next(self, parameters):
+        self.logger.info("browse next")
+        request = ua.BrowseNextRequest()
+        request.Parameters = parameters
+        data = self._uasocket.send_request(request)
+        response = ua.BrowseNextResponse.from_binary(data)
+        self.logger.debug(response)
+        response.ResponseHeader.ServiceResult.check()
+        return response.Parameters.Results
+
     def read(self, parameters):
         self.logger.info("read")
         request = ua.ReadRequest()
@@ -492,6 +504,16 @@ class UaClient(object):
         response.ResponseHeader.ServiceResult.check()
         return response.Results
 
+    def add_references(self, refs):
+        self.logger.info("add_references")
+        request = ua.AddReferencesRequest()
+        request.Parameters.ReferencesToAdd = refs
+        data = self._uasocket.send_request(request)
+        response = ua.AddReferencesResponse.from_binary(data)
+        self.logger.debug(response)
+        response.ResponseHeader.ServiceResult.check()
+        return response.Results
+
     def delete_nodes(self, params):
         self.logger.info("delete_nodes")
         request = ua.DeleteNodesRequest()
