@@ -5,7 +5,7 @@ and browse address space
 
 from opcua import ua
 from opcua.common import events
-import opcua.common 
+import opcua.common
 
 class Node(object):
 
@@ -29,7 +29,7 @@ class Node(object):
         elif isinstance(nodeid, int):
             self.nodeid = ua.NodeId(nodeid, 0)
         else:
-            raise ua.UaError("argument to node must be a NodeId object or a string defining a nodeid found {} of type {}".format(nodeid, type(nodeid)))
+            raise ua.UaError("argument to node must be a NodeId object or a string defining a nodeid found {0} of type {1}".format(nodeid, type(nodeid)))
 
     def __eq__(self, other):
         if isinstance(other, Node) and self.nodeid == other.nodeid:
@@ -40,7 +40,7 @@ class Node(object):
         return not self.__eq__(other)
 
     def __str__(self):
-        return "Node({})".format(self.nodeid)
+        return "Node({0})".format(self.nodeid)
     __repr__ = __str__
 
     def __hash__(self):
@@ -75,7 +75,7 @@ class Node(object):
         may not be convertible to VariantType
         """
         result = self.get_attribute(ua.AttributeIds.DataType)
-        return ua.DataType_to_VariantType(result.Value.Value)
+        return opcua.common.ua_utils.data_type_to_variant_type(Node(self.server, result.Value.Value))
 
     def get_access_level(self):
         """
@@ -308,6 +308,12 @@ class Node(object):
     def get_children_descriptions(self, refs=ua.ObjectIds.HierarchicalReferences, nodeclassmask=ua.NodeClass.Unspecified, includesubtypes=True):
         return self.get_references(refs, ua.BrowseDirection.Forward, nodeclassmask, includesubtypes)
 
+    def get_encoding_refs(self):
+        return self.get_referenced_nodes(ua.ObjectIds.HasEncoding, ua.BrowseDirection.Forward)
+
+    def get_description_refs(self):
+        return self.get_referenced_nodes(ua.ObjectIds.HasDescription, ua.BrowseDirection.Forward)
+
     def get_references(self, refs=ua.ObjectIds.References, direction=ua.BrowseDirection.Both, nodeclassmask=ua.NodeClass.Unspecified, includesubtypes=True):
         """
         returns references of the node based on specific filter defined with:
@@ -328,8 +334,21 @@ class Node(object):
         params = ua.BrowseParameters()
         params.View.Timestamp = ua.get_win_epoch()
         params.NodesToBrowse.append(desc)
+        params.RequestedMaxReferencesPerNode = 0
         results = self.server.browse(params)
-        return results[0].References
+
+        references = self._browse_next(results)
+        return references
+
+    def _browse_next(self, results):
+        references = results[0].References
+        while results[0].ContinuationPoint:
+            params = ua.BrowseNextParameters()
+            params.ContinuationPoints = [results[0].ContinuationPoint]
+            params.ReleaseContinuationPoints = False
+            results = self.server.browse_next(params)
+            references.extend(results[0].References)
+        return references
 
     def get_referenced_nodes(self, refs=ua.ObjectIds.References, direction=ua.BrowseDirection.Both, nodeclassmask=ua.NodeClass.Unspecified, includesubtypes=True):
         """
@@ -353,7 +372,7 @@ class Node(object):
             return None
         return references[0].NodeId
 
-    def get_path(self, max_length=20):
+    def get_path_as_string(self, max_length=20):
         """
         Attempt to find path of node from root node and return it as a list of strings.
         There might several possible paths to a node, this function will return one
@@ -362,14 +381,42 @@ class Node(object):
         Since address space may have circular references, a max length is specified
 
         """
-        path = [self.get_browse_name().to_string()]
+        path = self._get_path(max_length)
+        path = [ref.BrowseName.to_string() for ref in path]
+        path.append(self.get_browse_name().to_string())
+        return path
+
+    def get_path(self, max_length=20):
+        """
+        Attempt to find path of node from root node and return it as a list of Nodes.
+        There might several possible paths to a node, this function will return one
+        Some nodes may be missing references, so this method may
+        return an empty list
+        Since address space may have circular references, a max length is specified
+
+        """
+        path = self._get_path(max_length)
+        path = [Node(self.server, ref.NodeId) for ref in path]
+        path.append(self)
+        return path
+
+    def _get_path(self, max_length=20):
+        """
+        Attempt to find path of node from root node and return it as a list of Nodes.
+        There might several possible paths to a node, this function will return one
+        Some nodes may be missing references, so this method may
+        return an empty list
+        Since address space may have circular references, a max length is specified
+
+        """
+        path = []
         node = self
         while True:
             refs = node.get_references(refs=ua.ObjectIds.HierarchicalReferences, direction=ua.BrowseDirection.Inverse)
             if len(refs) > 0:
-                path.insert(0, refs[0].BrowseName.to_string())
+                path.insert(0, refs[0])
                 node = Node(self.server, refs[0].NodeId)
-                if len(path) >= max_length:
+                if len(path) >= (max_length -1):
                     return path
             else:
                 return path
@@ -525,30 +572,30 @@ class Node(object):
 
     def add_folder(self, nodeid, bname):
         return  opcua.common.manage_nodes.create_folder(self, nodeid, bname)
- 
+
     def add_object(self, nodeid, bname, objecttype=None):
         return opcua.common.manage_nodes.create_object(self, nodeid, bname, objecttype)
- 
+
     def add_variable(self, nodeid, bname, val, varianttype=None, datatype=None):
         return opcua.common.manage_nodes.create_variable(self, nodeid, bname, val, varianttype, datatype)
- 
+
     def add_object_type(self, nodeid, bname):
         return opcua.common.manage_nodes.create_object_type(self, nodeid, bname)
- 
+
     def add_variable_type(self, nodeid, bname, datatype):
         return opcua.common.manage_nodes.create_variable_type(self, nodeid, bname, datatype)
- 
+
     def add_data_type(self, nodeid, bname, description=None):
         return opcua.common.manage_nodes.create_data_type(self, nodeid, bname, description=None)
- 
+
     def add_property(self, nodeid, bname, val, varianttype=None, datatype=None):
         return opcua.common.manage_nodes.create_property(self, nodeid, bname, val, varianttype, datatype)
- 
+
     def add_method(self, *args):
         return opcua.common.manage_nodes.create_method(self, *args)
- 
+
     def add_reference_type(self, parent, nodeid, bname):
         return opcua.common.manage_nodes.create_reference_type(parent, nodeid, bname)
-    
-    def call_method(parent, methodid, *args):
-        return opcua.common.methods.call_method(parent, methodid, *args)
+
+    def call_method(self, methodid, *args):
+        return opcua.common.methods.call_method(self, methodid, *args)
