@@ -205,9 +205,15 @@ class MonitoredItemService(object):
                     self.isub.enqueue_datachange_event(mid, event, mdata.queue_size)
 
     def deadband_callback(self, values, flt):
-        if (values.get_old_value() is not None) or \
+        ua.DeadbandType.None_
+        if flt.DeadbandType == ua.DeadbandType.None_ or values.get_old_value() is None:
+            return True
+        elif flt.DeadbandType == ua.DeadbandType.Absolute and \
                 ((abs(values.get_current_value() - values.get_old_value())) > flt.DeadbandValue):
             return True
+        elif flt.DeadbandType == ua.DeadbandType.Percent:
+            self.logger.warn("DeadbandType Percent is not implemented !")
+            return True
         else:
             return False
 
@@ -263,7 +269,7 @@ class InternalSubscription(object):
         self._stopev = False
 
     def __str__(self):
-        return "Subscription(id:{})".format(self.data.SubscriptionId)
+        return "Subscription(id:{0})".format(self.data.SubscriptionId)
 
     def start(self):
         self.logger.debug("starting subscription %s", self.data.SubscriptionId)
@@ -304,7 +310,7 @@ class InternalSubscription(object):
             self._stopev = True
         result = None
         with self._lock:
-            if self.has_published_results():  
+            if self.has_published_results():
                 # FIXME: should we pop a publish request here? or we do not care?
                 self._publish_cycles_count += 1
                 result = self._pop_publish_result()
