@@ -1,29 +1,31 @@
 """
 implement ua datatypes
 """
+
 import logging
 import struct
-from enum import Enum, IntEnum, EnumMeta
+from enum import Enum, IntEnum
 from datetime import datetime
 import sys
 import os
 import uuid
 import re
 import itertools
-if sys.version_info.major > 2:
-    unicode = str
 
 from opcua.ua import ua_binary as uabin
 from opcua.ua import status_codes
 from opcua.ua import ObjectIds
-from opcua.common.uaerrors import UaError
-from opcua.common.uaerrors import UaStatusCodeError
-from opcua.common.uaerrors import UaStringParsingError
+from opcua.ua.uaerrors import UaError
+from opcua.ua.uaerrors import UaStatusCodeError
+from opcua.ua.uaerrors import UaStringParsingError
+from opcua.common.utils import Buffer
 
 
-logger = logging.getLogger('opcua.uaprotocol')
+logger = logging.getLogger(__name__)
 
 
+if sys.version_info.major > 2:
+    unicode = str
 def get_win_epoch():
     return uabin.win_epoch_to_datetime(0)
 
@@ -31,17 +33,19 @@ def get_win_epoch():
 class _FrozenClass(object):
 
     """
-    make it impossible to add members to a class.
-    This is a hack since I found out that most bugs are due to misspelling a variable in protocol
+    Make it impossible to add members to a class.
+    Not pythonic at all but we found out it prevents many many
+    bugs in use of protocol structures
     """
     _freeze = False
 
     def __setattr__(self, key, value):
         if self._freeze and not hasattr(self, key):
-            raise TypeError("Error adding member '{}' to class '{}', class is frozen, members are {}".format(
+            raise TypeError("Error adding member '{0}' to class '{1}', class is frozen, members are {2}".format(
                 key, self.__class__.__name__, self.__dict__.keys()))
         object.__setattr__(self, key, value)
 
+
 if "PYOPCUA_NO_TYPO_CHECK" in os.environ:
     # typo check is cpu consuming, but it will make debug easy.
     # if typo check is not need (in production), please set env PYOPCUA_NO_TYPO_CHECK.
@@ -165,30 +169,7 @@ class EventNotifier(_MaskEnum):
     HistoryWrite = 3
 
 
-class Guid(FrozenClass):
-
-    def __init__(self):
-        self.uuid = uuid.uuid4()
-        self._freeze = True
-
-    def to_binary(self):
-        return self.uuid.bytes
-
-    def __hash__(self):
-        return hash(self.uuid.bytes)
-
-    @staticmethod
-    def from_binary(data):
-        g = Guid()
-        g.uuid = uuid.UUID(bytes=data.read(16))
-        return g
-
-    def __eq__(self, other):
-        return isinstance(other, Guid) and self.uuid == other.uuid
-
-
 class StatusCode(FrozenClass):
-
     """
     :ivar value:
     :vartype value: int
@@ -236,7 +217,7 @@ class StatusCode(FrozenClass):
             return True
 
     def __str__(self):
-        return 'StatusCode({})'.format(self.name)
+        return 'StatusCode({0})'.format(self.name)
     __repr__ = __str__
 
     def __eq__(self, other):
@@ -246,7 +227,7 @@ class StatusCode(FrozenClass):
         return not self.__eq__(other)
 
 
-class NodeIdType(Enum):
+class NodeIdType(IntEnum):
     TwoByte = 0
     FourByte = 1
     Numeric = 2
@@ -256,7 +237,6 @@ class NodeIdType(Enum):
 
 
 class NodeId(FrozenClass):
-
     """
     NodeId Object
 
@@ -275,8 +255,9 @@ class NodeId(FrozenClass):
     :ivar ServerIndex:
     :vartype ServerIndex: Int
     """
-
+    
     def __init__(self, identifier=None, namespaceidx=0, nodeidtype=None):
+
         self.Identifier = identifier
         self.NamespaceIndex = namespaceidx
         self.NodeIdType = nodeidtype
@@ -296,23 +277,30 @@ class NodeId(FrozenClass):
                 self.NodeIdType = NodeIdType.String
             elif isinstance(self.Identifier, bytes):
                 self.NodeIdType = NodeIdType.ByteString
+            elif isinstance(self.Identifier, uuid.UUID):
+                self.NodeIdType = NodeIdType.Guid
             else:
                 raise UaError("NodeId: Could not guess type of NodeId, set NodeIdType")
 
-    def __key(self):
-        if self.NodeIdType in (NodeIdType.TwoByte, NodeIdType.FourByte, NodeIdType.Numeric):  # twobyte, fourbyte and numeric may represent the same node
-            return self.NamespaceIndex, self.Identifier
-        else:
-            return self.NodeIdType, self.NamespaceIndex, self.Identifier
+    def _key(self):
+        if self.NodeIdType in (NodeIdType.TwoByte, NodeIdType.FourByte, NodeIdType.Numeric): 
+            # twobyte, fourbyte and numeric may represent the same node
+            return (NodeIdType.Numeric, self.NamespaceIndex, self.Identifier)
+        return (self.NodeIdType, self.NamespaceIndex, self.Identifier)
 
     def __eq__(self, node):
-        return isinstance(node, NodeId) and self.__key() == node.__key()
+        return isinstance(node, NodeId) and self._key() == node._key()
 
     def __ne__(self, other):
         return not self.__eq__(other)
 
     def __hash__(self):
-        return hash(self.__key())
+        return hash(self._key())
+
+    def __lt__(self, other):
+        if not isinstance(other, NodeId):
+            raise AttributeError("Can only compare to NodeId")
+        return self._key() < other._key()
 
     def is_null(self):
         if self.NamespaceIndex != 0:
@@ -331,7 +319,7 @@ class NodeId(FrozenClass):
         try:
             return NodeId._from_string(string)
         except ValueError as ex:
-            raise UaStringParsingError("Error parsing string {}".format(string), ex)
+            raise UaStringParsingError("Error parsing string {0}".format(string), ex)
 
     @staticmethod
     def _from_string(string):
@@ -375,7 +363,7 @@ class NodeId(FrozenClass):
     def to_string(self):
         string = ""
         if self.NamespaceIndex != 0:
-            string += "ns={};".format(self.NamespaceIndex)
+            string += "ns={0};".format(self.NamespaceIndex)
         ntype = None
         if self.NodeIdType == NodeIdType.Numeric:
             ntype = "i"
@@ -389,15 +377,15 @@ class NodeId(FrozenClass):
             ntype = "g"
         elif self.NodeIdType == NodeIdType.ByteString:
             ntype = "b"
-        string += "{}={}".format(ntype, self.Identifier)
+        string += "{0}={1}".format(ntype, self.Identifier)
         if self.ServerIndex:
             string = "srv=" + str(self.ServerIndex) + string
         if self.NamespaceUri:
-            string += "nsu={}".format(self.NamespaceUri)
+            string += "nsu={0}".format(self.NamespaceUri)
         return string
 
     def __str__(self):
-        return "{}NodeId({})".format(self.NodeIdType.name, self.to_string())
+        return "{0}NodeId({1})".format(self.NodeIdType.name, self.to_string())
     __repr__ = __str__
 
     def to_binary(self):
@@ -413,6 +401,9 @@ class NodeId(FrozenClass):
         elif self.NodeIdType == NodeIdType.ByteString:
             return struct.pack("<BH", self.NodeIdType.value, self.NamespaceIndex) + \
                 uabin.Primitives.Bytes.pack(self.Identifier)
+        elif self.NodeIdType == NodeIdType.Guid:
+            return struct.pack("<BH", self.NodeIdType.value, self.NamespaceIndex) + \
+                   uabin.Primitives.Guid.pack(self.Identifier)
         else:
             return struct.pack("<BH", self.NodeIdType.value, self.NamespaceIndex) + \
                 self.Identifier.to_binary()
@@ -438,7 +429,7 @@ class NodeId(FrozenClass):
             nid.Identifier = uabin.Primitives.Bytes.unpack(data)
         elif nid.NodeIdType == NodeIdType.Guid:
             nid.NamespaceIndex = uabin.Primitives.UInt16.unpack(data)
-            nid.Identifier = Guid.from_binary(data)
+            nid.Identifier = uabin.Primitives.Guid.unpack(data)
         else:
             raise UaError("Unknown NodeId encoding: " + str(nid.NodeIdType))
 
@@ -490,10 +481,9 @@ ExpandedNodeId = NodeId
 
 
 class QualifiedName(FrozenClass):
-
-    '''
+    """
     A string qualified with a namespace index.
-    '''
+    """
 
     def __init__(self, name=None, namespaceidx=0):
         if not isinstance(namespaceidx, int):
@@ -503,7 +493,7 @@ class QualifiedName(FrozenClass):
         self._freeze = True
 
     def to_string(self):
-        return "{}:{}".format(self.NamespaceIndex, self.Name)
+        return "{0}:{1}".format(self.NamespaceIndex, self.Name)
 
     @staticmethod
     def from_string(string):
@@ -512,7 +502,7 @@ class QualifiedName(FrozenClass):
                 idx, name = string.split(":", 1)
                 idx = int(idx)
             except (TypeError, ValueError) as ex:
-                raise UaStringParsingError("Error parsing string {}".format(string), ex)
+                raise UaStringParsingError("Error parsing string {0}".format(string), ex)
         else:
             idx = 0
             name = string
@@ -539,27 +529,26 @@ class QualifiedName(FrozenClass):
 
     def __lt__(self, other):
         if not isinstance(other, QualifiedName):
-            raise TypeError("Cannot compare QualifiedName and {}".format(other))
+            raise TypeError("Cannot compare QualifiedName and {0}".format(other))
         if self.NamespaceIndex == other.NamespaceIndex:
             return self.Name < other.Name
         else:
             return self.NamespaceIndex < other.NamespaceIndex
 
     def __str__(self):
-        return 'QualifiedName({}:{})'.format(self.NamespaceIndex, self.Name)
+        return 'QualifiedName({0}:{1})'.format(self.NamespaceIndex, self.Name)
 
     __repr__ = __str__
 
 
 class LocalizedText(FrozenClass):
-
-    '''
+    """
     A string qualified with a namespace index.
-    '''
+    """
 
     ua_types = {
-        "Text": "Bytes",
-        "Locale": "Bytes"
+        "Text": "ByteString",
+        "Locale": "ByteString"
     }
 
     def __init__(self, text=None):
@@ -617,18 +606,20 @@ class LocalizedText(FrozenClass):
 
 
 class ExtensionObject(FrozenClass):
-
-    '''
-
+    """
     Any UA object packed as an ExtensionObject
 
-
     :ivar TypeId:
     :vartype TypeId: NodeId
     :ivar Body:
     :vartype Body: bytes
+    """
 
-    '''
+    ua_types = {
+        "TypeId": "NodeId",
+        "Encoding": "Byte",
+        "Body": "ByteString"
+    }
 
     def __init__(self):
         self.TypeId = NodeId()
@@ -639,7 +630,7 @@ class ExtensionObject(FrozenClass):
     def to_binary(self):
         packet = []
         if self.Body:
-            self.Encoding |= (1 << 0)
+            self.Encoding = 0x01
         packet.append(self.TypeId.to_binary())
         packet.append(uabin.Primitives.UInt8.pack(self.Encoding))
         if self.Body:
@@ -658,7 +649,7 @@ class ExtensionObject(FrozenClass):
     @staticmethod
     def from_object(obj):
         ext = ExtensionObject()
-        oid = getattr(ObjectIds, "{}_Encoding_DefaultBinary".format(obj.__class__.__name__))
+        oid = getattr(ObjectIds, "{0}_Encoding_DefaultBinary".format(obj.__class__.__name__))
         ext.TypeId = FourByteNodeId(oid)
         ext.Body = obj.to_binary()
         return ext
@@ -671,8 +662,7 @@ class ExtensionObject(FrozenClass):
 
 
 class VariantType(Enum):
-
-    '''
+    """
     The possible types of a variant.
 
     :ivar Null:
@@ -701,10 +691,8 @@ class VariantType(Enum):
     :ivar DataValue:
     :ivar Variant:
     :ivar DiagnosticInfo:
+    """
 
-
-
-    '''
     Null = 0
     Boolean = 1
     SByte = 2
@@ -745,10 +733,10 @@ class VariantTypeCustom(object):
         self.name = "Custom"
         self.value = val
         if self.value > 0b00111111:
-            raise UaError("Cannot create VariantType. VariantType must be {} > x > {}, received {}".format(0b111111, 25, val))
+            raise UaError("Cannot create VariantType. VariantType must be {0} > x > {1}, received {2}".format(0b111111, 25, val))
 
     def __str__(self):
-        return "VariantType.Custom:{}".format(self.value)
+        return "VariantType.Custom:{0}".format(self.value)
     __repr__ = __str__
 
     def __eq__(self, other):
@@ -756,7 +744,6 @@ class VariantTypeCustom(object):
 
 
 class Variant(FrozenClass):
-
     """
     Create an OPC-UA Variant object.
     if no argument a Null Variant is created.
@@ -767,19 +754,34 @@ class Variant(FrozenClass):
     :vartype Value: Any supported type
     :ivar VariantType:
     :vartype VariantType: VariantType
+    :ivar Dimension:
+    :vartype Dimensions: The length of each dimensions. Usually guessed from value.
+    :ivar is_array:
+    :vartype is_array: If the variant is an array. Usually guessed from value.
     """
 
-    def __init__(self, value=None, varianttype=None, dimensions=None):
+    def __init__(self, value=None, varianttype=None, dimensions=None, is_array=None):
         self.Value = value
         self.VariantType = varianttype
         self.Dimensions = dimensions
+        self.is_array = is_array
+        if self.is_array is None:
+            if isinstance(value, (list, tuple)):
+                self.is_array = True
+            else:
+                self.is_array = False
         self._freeze = True
         if isinstance(value, Variant):
             self.Value = value.Value
             self.VariantType = value.VariantType
         if self.VariantType is None:
             self.VariantType = self._guess_type(self.Value)
-        if self.Dimensions is None and type(self.Value) in (list, tuple):
+        if self.Value is None and not self.is_array and self.VariantType not in (
+                VariantType.Null,
+                VariantType.String,
+                VariantType.DateTime):
+            raise UaError("Non array Variant of type {0} cannot have value None".format(self.VariantType))
+        if self.Dimensions is None and isinstance(self.Value, (list, tuple)):
             dims = get_shape(self.Value)
             if len(dims) > 1:
                 self.Dimensions = dims
@@ -797,7 +799,7 @@ class Variant(FrozenClass):
             error_val = val
         while isinstance(val, (list, tuple)):
             if len(val) == 0:
-                raise UaError("could not guess UA type of variable {}".format(error_val))
+                raise UaError("could not guess UA type of variable {0}".format(error_val))
             val = val[0]
         if val is None:
             return VariantType.Null
@@ -805,6 +807,8 @@ class Variant(FrozenClass):
             return VariantType.Boolean
         elif isinstance(val, float):
             return VariantType.Double
+        elif isinstance(val, IntEnum):
+            return VariantType.Int32
         elif isinstance(val, int):
             return VariantType.Int64
         elif type(val) in (str, unicode):
@@ -813,6 +817,8 @@ class Variant(FrozenClass):
             return VariantType.ByteString
         elif isinstance(val, datetime):
             return VariantType.DateTime
+        elif isinstance(val, uuid.UUID):
+            return VariantType.Guid
         else:
             if isinstance(val, object):
                 try:
@@ -820,19 +826,20 @@ class Variant(FrozenClass):
                 except AttributeError:
                     return VariantType.ExtensionObject
             else:
-                raise UaError("Could not guess UA type of {} with type {}, specify UA type".format(val, type(val)))
+                raise UaError("Could not guess UA type of {0} with type {1}, specify UA type".format(val, type(val)))
 
     def __str__(self):
-        return "Variant(val:{!s},type:{})".format(self.Value, self.VariantType)
+        return "Variant(val:{0!s},type:{1})".format(self.Value, self.VariantType)
     __repr__ = __str__
 
     def to_binary(self):
         b = []
         encoding = self.VariantType.value & 0b111111
-        if type(self.Value) in (list, tuple):
+        if self.is_array or type(self.Value) in (list, tuple):
+            self.is_array = True
+            encoding = uabin.set_bit(encoding, 7)
             if self.Dimensions is not None:
                 encoding = uabin.set_bit(encoding, 6)
-            encoding = uabin.set_bit(encoding, 7)
             b.append(uabin.Primitives.UInt8.pack(encoding))
             b.append(uabin.pack_uatype_array(self.VariantType, flatten(self.Value)))
             if self.Dimensions is not None:
@@ -846,20 +853,19 @@ class Variant(FrozenClass):
     @staticmethod
     def from_binary(data):
         dimensions = None
+        array = False
         encoding = ord(data.read(1))
         int_type = encoding & 0b00111111
-        vtype = DataType_to_VariantType(int_type)
-        if vtype == VariantType.Null:
-            return Variant(None, vtype, encoding)
+        vtype = datatype_to_varianttype(int_type)
         if uabin.test_bit(encoding, 7):
             value = uabin.unpack_uatype_array(vtype, data)
+            array = True
         else:
             value = uabin.unpack_uatype(vtype, data)
         if uabin.test_bit(encoding, 6):
             dimensions = uabin.unpack_uatype_array(VariantType.Int32, data)
             value = reshape(value, dimensions)
-
-        return Variant(value, vtype, dimensions)
+        return Variant(value, vtype, dimensions, is_array=array)
 
 
 def reshape(flat, dims):
@@ -893,7 +899,9 @@ def flatten_and_get_shape(mylist):
 
 
 def flatten(mylist):
-    if len(mylist) == 0:
+    if mylist is None:
+        return None
+    elif len(mylist) == 0:
         return mylist
     while isinstance(mylist[0], (list, tuple)):
         mylist = [item for sublist in mylist for item in sublist]
@@ -913,24 +921,27 @@ def get_shape(mylist):
 
 
 class XmlElement(FrozenClass):
-    '''
+    """
     An XML element encoded as an UTF-8 string.
-    '''
+    """
 
-    def __init__(self, binary=None):
+    def __init__(self, value=None, binary=None):
         if binary is not None:
             self._binary_init(binary)
             self._freeze = True
             return
-        self.Value = []
+        self.Value = value
         self._freeze = True
 
     def to_binary(self):
         return uabin.Primitives.String.pack(self.Value)
 
+    def __eq__(self, other):
+        return isinstance(other, XmlElement) and self.Value == other.Value
+
     @staticmethod
     def from_binary(data):
-        return XmlElement(data)
+        return XmlElement(binary=data)
 
     def _binary_init(self, data):
         self.Value = uabin.Primitives.String.unpack(data)
@@ -942,8 +953,7 @@ class XmlElement(FrozenClass):
 
 
 class DataValue(FrozenClass):
-
-    '''
+    """
     A value with an associated timestamp, and quality.
     Automatically generated from xml , copied and modified here to fix errors in xml spec
 
@@ -959,8 +969,7 @@ class DataValue(FrozenClass):
     :vartype ServerTimestamp: datetime
     :ivar ServerPicoseconds:
     :vartype ServerPicoseconds: int
-
-    '''
+    """
 
     def __init__(self, variant=None, status=None):
         self.Encoding = 0
@@ -1030,28 +1039,28 @@ class DataValue(FrozenClass):
         return obj
 
     def __str__(self):
-        s = 'DataValue(Value:{}'.format(self.Value)
+        s = 'DataValue(Value:{0}'.format(self.Value)
         if self.StatusCode is not None:
-            s += ', StatusCode:{}'.format(self.StatusCode)
+            s += ', StatusCode:{0}'.format(self.StatusCode)
         if self.SourceTimestamp is not None:
-            s += ', SourceTimestamp:{}'.format(self.SourceTimestamp)
+            s += ', SourceTimestamp:{0}'.format(self.SourceTimestamp)
         if self.ServerTimestamp is not None:
-            s += ', ServerTimestamp:{}'.format(self.ServerTimestamp)
+            s += ', ServerTimestamp:{0}'.format(self.ServerTimestamp)
         if self.SourcePicoseconds is not None:
-            s += ', SourcePicoseconds:{}'.format(self.SourcePicoseconds)
+            s += ', SourcePicoseconds:{0}'.format(self.SourcePicoseconds)
         if self.ServerPicoseconds is not None:
-            s += ', ServerPicoseconds:{}'.format(self.ServerPicoseconds)
+            s += ', ServerPicoseconds:{0}'.format(self.ServerPicoseconds)
         s += ')'
         return s
 
     __repr__ = __str__
 
 
-def DataType_to_VariantType(int_type):
+def datatype_to_varianttype(int_type):
     """
     Takes a NodeId or int and return a VariantType
     This is only supported if int_type < 63 due to VariantType encoding
-    At low level we do not have access to address space thus decodig is limited
+    At low level we do not have access to address space thus decoding is limited
     a better version of this method can be find in ua_utils.py
     """
     if isinstance(int_type, NodeId):
@@ -1061,3 +1070,116 @@ def DataType_to_VariantType(int_type):
         return VariantType(int_type)
     else:
         return VariantTypeCustom(int_type)
+
+
+def get_default_value(vtype):
+    """
+    Given a variant type return default value for this type
+    """
+    if vtype == VariantType.Null:
+        return None
+    elif vtype == VariantType.Boolean:
+        return False
+    elif vtype in (VariantType.SByte, VariantType.Byte):
+        return 0
+    elif vtype == VariantType.ByteString:
+        return b""
+    elif 4 <= vtype.value <= 9:
+        return 0
+    elif vtype in (VariantType.Float, VariantType.Double):
+        return 0.0
+    elif vtype == VariantType.String:
+        return None  # a string can be null
+    elif vtype == VariantType.DateTime:
+        return datetime.utcnow()
+    elif vtype == VariantType.Guid:
+        return uuid.uuid4()
+    elif vtype == VariantType.XmlElement:
+        return None  #Not sure this is correct
+    elif vtype == VariantType.NodeId:
+        return NodeId()
+    elif vtype == VariantType.ExpandedNodeId:
+        return NodeId()
+    elif vtype == VariantType.StatusCode:
+        return StatusCode()
+    elif vtype == VariantType.QualifiedName:
+        return QualifiedName()
+    elif vtype == VariantType.LocalizedText:
+        return LocalizedText()
+    elif vtype == VariantType.ExtensionObject:
+        return ExtensionObject()
+    elif vtype == VariantType.DataValue:
+        return DataValue()
+    elif vtype == VariantType.Variant:
+        return Variant()
+    else:
+        raise RuntimeError("function take a uatype as argument, got:", vtype)
+
+
+# These dictionnaries are used to register extensions classes for automatic
+# decoding and encoding
+extension_object_classes = {}
+extension_object_ids = {}
+
+
+def register_extension_object(name, nodeid, class_type):
+    """
+    """
+    logger.warning("registring new extension object: %s %s %s", name, nodeid, class_type)
+    extension_object_classes[nodeid] = class_type
+    extension_object_ids[name] = nodeid
+
+
+def extensionobject_from_binary(data):
+    """
+    Convert binary-coded ExtensionObject to a Python object.
+    Returns an object, or None if TypeId is zero
+    """
+    typeid = NodeId.from_binary(data)
+    Encoding = ord(data.read(1))
+    body = None
+    if Encoding & (1 << 0):
+        length = uabin.Primitives.Int32.unpack(data)
+        if length < 1:
+            body = Buffer(b"")
+        else:
+            body = data.copy(length)
+            data.skip(length)
+    if typeid.Identifier == 0:
+        return None
+    elif typeid in extension_object_classes:
+        klass = extension_object_classes[typeid]
+        if body is None:
+            raise UaError("parsing ExtensionObject {0} without data".format(klass.__name__))
+        return klass.from_binary(body)
+    else:
+        e = ExtensionObject()
+        e.TypeId = typeid
+        e.Encoding = Encoding
+        if body is not None:
+            e.Body = body.read(len(body))
+        return e
+
+
+def extensionobject_to_binary(obj):
+    """
+    Convert Python object to binary-coded ExtensionObject.
+    If obj is None, convert to empty ExtensionObject (TypeId = 0, no Body).
+    Returns a binary string
+    """
+    if isinstance(obj, ExtensionObject):
+        return obj.to_binary()
+    if obj is None:
+        TypeId = NodeId()
+        Encoding = 0
+        Body = None
+    else:
+        TypeId = extension_object_ids[obj.__class__.__name__]
+        Encoding = 0x01
+        Body = obj.to_binary()
+    packet = []
+    packet.append(TypeId.to_binary())
+    packet.append(uabin.Primitives.UInt8.pack(Encoding))
+    if Body:
+        packet.append(uabin.Primitives.Bytes.pack(Body))
+    return b''.join(packet)
