@@ -5,11 +5,12 @@ from datetime import datetime
 from datetime import timedelta
 import math
 
-import opcua
 from opcua import ua
+from opcua import Node
 from opcua import uamethod
 from opcua import instantiate
 from opcua import copy_node
+from opcua.common import ua_utils
 
 
 def add_server_methods(srv):
@@ -539,15 +540,27 @@ class CommonTests(object):
         self.assertEqual(p.get_parent(), o)
         self.assertEqual(p.get_type_definition().Identifier, ua.ObjectIds.PropertyType)
 
-    def test_path(self):
+    def test_path_string(self):
         o = self.opc.nodes.objects.add_folder(1, "titif").add_object(3, "opath")
-        path = o.get_path()
+        path = o.get_path_as_string()
         self.assertEqual(["0:Root", "0:Objects", "1:titif", "3:opath"], path)
-        path = o.get_path(2)
+        path = o.get_path_as_string(2)
         self.assertEqual(["1:titif", "3:opath"], path)
-        self.opc.get_node("i=27")
-        path = self.opc.get_node("i=13387").get_path()
-        self.assertEqual(['0:BaseObjectType', '0:FolderType', '0:FileDirectoryType', '0:CreateDirectory'], path) # FIXME this is wrong in our server! BaseObjectType is missing an inverse reference to its parent! seems xml definition is wrong
+        path = self.opc.get_node("i=13387").get_path_as_string()
+        # FIXME this is wrong in our server! BaseObjectType is missing an inverse reference to its parent! seems xml definition is wrong
+        self.assertEqual(['0:BaseObjectType', '0:FolderType', '0:FileDirectoryType', '0:CreateDirectory'], path) 
+
+    def test_path(self):
+        of = self.opc.nodes.objects.add_folder(1, "titif")
+        op = of.add_object(3, "opath")
+        path = op.get_path()
+        self.assertEqual([self.opc.nodes.root, self.opc.nodes.objects, of, op], path)
+        path = op.get_path(2)
+        self.assertEqual([of, op], path)
+        target = self.opc.get_node("i=13387")
+        path = target.get_path()
+        # FIXME this is wrong in our server! BaseObjectType is missing an inverse reference to its parent! seems xml definition is wrong
+        self.assertEqual([self.opc.nodes.base_object_type, self.opc.nodes.folder_type, self.opc.get_node(ua.ObjectIds.FileDirectoryType), target], path)  
 
     def test_get_endpoints(self):
         endpoints = self.opc.get_endpoints()
@@ -608,8 +621,29 @@ class CommonTests(object):
         prop1 = mydevicederived.get_child(["0:sensorx_id"])
         var1 = mydevicederived.get_child(["0:childparam"])
         var_parent = mydevicederived.get_child(["0:sensor"])
-        prop_parent = mydevicederived.get_child(["0:sensor_id"])        
+        prop_parent = mydevicederived.get_child(["0:sensor_id"])
 
+    def test_instantiate_string_nodeid(self):
+        # Create device type
+        dev_t = self.opc.nodes.base_object_type.add_object_type(0, "MyDevice2")
+        v_t = dev_t.add_variable(0, "sensor", 1.0)
+        p_t = dev_t.add_property(0, "sensor_id", "0340")
+        ctrl_t = dev_t.add_object(0, "controller")
+        prop_t = ctrl_t.add_property(0, "state", "Running")
+
+        # instanciate device
+        nodes = instantiate(self.opc.nodes.objects, dev_t, nodeid=ua.NodeId("InstDevice", 2, ua.NodeIdType.String), bname="2:InstDevice")
+        mydevice = nodes[0]
+
+        self.assertEqual(mydevice.get_node_class(), ua.NodeClass.Object)
+        self.assertEqual(mydevice.get_type_definition(), dev_t.nodeid)
+        obj = mydevice.get_child(["0:controller"])
+        obj_nodeid_ident = obj.nodeid.Identifier
+        prop = mydevice.get_child(["0:controller", "0:state"])
+        self.assertEqual(obj_nodeid_ident, "InstDevice.controller")
+        self.assertEqual(prop.get_type_definition().Identifier, ua.ObjectIds.PropertyType)
+        self.assertEqual(prop.get_value(), "Running")
+        self.assertNotEqual(prop.nodeid, prop_t.nodeid)
 
     def test_variable_with_datatype(self):
         v1 = self.opc.nodes.objects.add_variable(3, 'VariableEnumType1', ua.ApplicationType.ClientAndServer, datatype=ua.NodeId(ua.ObjectIds.ApplicationType))
@@ -634,4 +668,56 @@ class CommonTests(object):
         self.assertEqual(myvar.get_data_type(), myenum_type.nodeid)
         myvar.set_value(ua.LocalizedText("String2"))
 
+    def test_supertypes(self):
+        nint32 = self.opc.get_node(ua.ObjectIds.Int32)
+        node = ua_utils.get_node_supertype(nint32)
+        self.assertEqual(node, self.opc.get_node(ua.ObjectIds.Integer))
 
+        nodes = ua_utils.get_node_supertypes(nint32)
+        self.assertEqual(nodes[1], self.opc.get_node(ua.ObjectIds.Number))
+        self.assertEqual(nodes[0], self.opc.get_node(ua.ObjectIds.Integer))
+
+        # test custom
+        dtype = nint32.add_data_type(0, "MyCustomDataType")
+        node = ua_utils.get_node_supertype(dtype)
+        self.assertEqual(node, nint32)
+
+        dtype2 = dtype.add_data_type(0, "MyCustomDataType2")
+        node = ua_utils.get_node_supertype(dtype2)
+        self.assertEqual(node, dtype)
+
+    def test_base_data_type(self):
+        nint32 = self.opc.get_node(ua.ObjectIds.Int32)
+        dtype = nint32.add_data_type(0, "MyCustomDataType")
+        dtype2 = dtype.add_data_type(0, "MyCustomDataType2")
+        self.assertEqual(ua_utils.get_base_data_type(dtype), nint32)
+        self.assertEqual(ua_utils.get_base_data_type(dtype2), nint32)
+
+        ext = self.opc.nodes.objects.add_variable(0, "MyExtensionObject", ua.Argument())
+        d = ext.get_data_type()
+        d = self.opc.get_node(d)
+        self.assertEqual(ua_utils.get_base_data_type(d), self.opc.get_node(ua.ObjectIds.Structure))
+        self.assertEqual(ua_utils.data_type_to_variant_type(d), ua.VariantType.ExtensionObject)
+
+    def test_data_type_to_variant_type(self):
+        test_data = {
+            ua.ObjectIds.Boolean: ua.VariantType.Boolean,
+            ua.ObjectIds.Byte: ua.VariantType.Byte,
+            ua.ObjectIds.String: ua.VariantType.String,
+            ua.ObjectIds.Int32: ua.VariantType.Int32,
+            ua.ObjectIds.UInt32: ua.VariantType.UInt32,
+            ua.ObjectIds.NodeId: ua.VariantType.NodeId,
+            ua.ObjectIds.LocalizedText: ua.VariantType.LocalizedText,
+            ua.ObjectIds.Structure: ua.VariantType.ExtensionObject,
+            ua.ObjectIds.EnumValueType: ua.VariantType.ExtensionObject,
+            ua.ObjectIds.Enumeration: ua.VariantType.Int32,  # enumeration
+            ua.ObjectIds.AttributeWriteMask: ua.VariantType.Int32,  # enumeration
+            ua.ObjectIds.AxisScaleEnumeration: ua.VariantType.Int32  # enumeration
+            }
+        for dt, vdt in test_data.items():
+            self.assertEqual(ua_utils.data_type_to_variant_type(self.opc.get_node(ua.NodeId(dt))), vdt)
+
+    def test_variant_intenum(self):
+        ase = ua.AxisScaleEnumeration(ua.AxisScaleEnumeration.Linear)  # Just pick an existing IntEnum class
+        vAse = ua.Variant(ase)
+        self.assertEqual(vAse.VariantType, ua.VariantType.Int32)
