@@ -4,6 +4,7 @@ import shelve
 import time
 
 from tests_common import CommonTests, add_server_methods
+from tests_xml import XmlTests
 from tests_subscriptions import SubscriptionTests
 from datetime import timedelta, datetime
 from tempfile import NamedTemporaryFile
@@ -13,35 +14,36 @@ from opcua import Server
 from opcua import Client
 from opcua import ua
 from opcua import uamethod
-from opcua.common.event_objects import BaseEvent, AuditEvent
+from opcua.common.event_objects import BaseEvent, AuditEvent, AuditChannelEvent, AuditSecurityEvent, AuditOpenSecureChannelEvent
+from opcua.common import ua_utils
 
 
 port_num = 48540
 port_discovery = 48550
 
 
-class TestServer(unittest.TestCase, CommonTests, SubscriptionTests):
+class TestServer(unittest.TestCase, CommonTests, SubscriptionTests, XmlTests):
 
     '''
     Run common tests on server side
     Tests that can only be run on server side must be defined here
     '''
     @classmethod
-    def setUpClass(self):
-        self.srv = Server()
-        self.srv.set_endpoint('opc.tcp://localhost:%d' % port_num)
-        add_server_methods(self.srv)
-        self.srv.start()
-        self.opc = self.srv
-        self.discovery = Server()
-        self.discovery.set_application_uri("urn:freeopcua:python:discovery")
-        self.discovery.set_endpoint('opc.tcp://localhost:%d' % port_discovery)
-        self.discovery.start()
+    def setUpClass(cls):
+        cls.srv = Server()
+        cls.srv.set_endpoint('opc.tcp://localhost:{0:d}'.format(port_num))
+        add_server_methods(cls.srv)
+        cls.srv.start()
+        cls.opc = cls.srv
+        cls.discovery = Server()
+        cls.discovery.set_application_uri("urn:freeopcua:python:discovery")
+        cls.discovery.set_endpoint('opc.tcp://localhost:{0:d}'.format(port_discovery))
+        cls.discovery.start()
 
     @classmethod
-    def tearDownClass(self):
-        self.srv.stop()
-        self.discovery.stop()
+    def tearDownClass(cls):
+        cls.srv.stop()
+        cls.discovery.stop()
 
     def test_discovery(self):
         client = Client(self.discovery.endpoint.geturl())
@@ -51,7 +53,7 @@ class TestServer(unittest.TestCase, CommonTests, SubscriptionTests):
             new_app_uri = "urn:freeopcua:python:server:test_discovery"
             self.srv.application_uri = new_app_uri
             self.srv.register_to_discovery(self.discovery.endpoint.geturl(), 0)
-            time.sleep(0.1) # let server register registration
+            time.sleep(0.1)  # let server register registration
             new_servers = client.find_servers()
             self.assertEqual(len(new_servers) - len(servers) , 1)
             self.assertFalse(new_app_uri in [s.ApplicationUri for s in servers])
@@ -70,7 +72,7 @@ class TestServer(unittest.TestCase, CommonTests, SubscriptionTests):
             new_app_uri2 = "urn:freeopcua:python:test_discovery2"
             self.srv.application_uri = new_app_uri2
             self.srv.register_to_discovery(self.discovery.endpoint.geturl(), period=0)
-            time.sleep(0.1) # let server register registration
+            time.sleep(0.1)  # let server register registration
             new_servers = client.find_servers()
             self.assertEqual(len(new_servers) - len(servers) , 2)
             self.assertFalse(new_app_uri1 in [s.ApplicationUri for s in servers])
@@ -102,8 +104,8 @@ class TestServer(unittest.TestCase, CommonTests, SubscriptionTests):
         print("SERVERS 2", servers)
         self.assertEqual(len(servers), 2)
     """
-    #def test_register_server2(self):
-        #servers = self.opc.register_server()
+    # def test_register_server2(self):
+        # servers = self.opc.register_server()
 
     def test_register_namespace(self):
         uri = 'http://mycustom.Namespace.com'
@@ -136,31 +138,6 @@ class TestServer(unittest.TestCase, CommonTests, SubscriptionTests):
         result = o.call_method(v, ua.Variant(2.1))
         self.assertEqual(result, 4.2)
 
-    def test_xml_import(self):
-        self.srv.import_xml("tests/custom_nodes.xml")
-        o = self.opc.get_objects_node()
-        v = o.get_child(["MyXMLFolder", "MyXMLObject", "MyXMLVariable"])
-        val = v.get_value()
-        self.assertEqual(val, "StringValue")
-
-        node_path = ["Types", "DataTypes", "BaseDataType", "Enumeration",
-                     "1:MyEnum", "0:EnumStrings"]
-        o = self.opc.get_root_node().get_child(node_path)
-        self.assertEqual(len(o.get_value()), 3)
-
-        # Check if method is imported
-        node_path = ["Types", "ObjectTypes", "BaseObjectType",
-                     "1:MyObjectType", "1:MyMethod"]
-        o = self.opc.get_root_node().get_child(node_path)
-        self.assertEqual(len(o.get_referenced_nodes()), 4)
-
-        # Check if InputArgs are imported and can be read
-        node_path = ["Types", "ObjectTypes", "BaseObjectType",
-                     "1:MyObjectType", "1:MyMethod", "InputArguments"]
-        o = self.opc.get_root_node().get_child(node_path)
-        input_arg = o.get_data_value().Value.Value[0]
-        self.assertEqual(input_arg.Name, 'Context')
-
     def test_historize_variable(self):
         o = self.opc.get_objects_node()
         var = o.add_variable(3, "test_hist", 1.0)
@@ -215,6 +192,23 @@ class TestServer(unittest.TestCase, CommonTests, SubscriptionTests):
         self.assertEqual(ev.ServerId, None)
         self.assertEqual(ev.ClientAuditEntryId, None)
         self.assertEqual(ev.ClientUserId, None)
+        
+    def test_get_event_from_type_node_MultiInhereted_AuditOpenSecureChannelEvent(self):
+        ev = opcua.common.events.get_event_obj_from_type_node(opcua.Node(self.opc.iserver.isession, ua.NodeId(ua.ObjectIds.AuditOpenSecureChannelEventType)))
+        self.assertIsNot(ev, None)
+        self.assertIsInstance(ev, BaseEvent)
+        self.assertIsInstance(ev, AuditEvent)
+        self.assertIsInstance(ev, AuditSecurityEvent)
+        self.assertIsInstance(ev, AuditChannelEvent)
+        self.assertIsInstance(ev, AuditOpenSecureChannelEvent)
+        self.assertEqual(ev.EventType, ua.NodeId(ua.ObjectIds.AuditOpenSecureChannelEventType))
+        self.assertEqual(ev.Severity, 1),
+        self.assertEqual(ev.ClientCertificate, None)
+        self.assertEqual(ev.ClientCertificateThumbprint, None)
+        self.assertEqual(ev.RequestType, None)
+        self.assertEqual(ev.SecurityPolicyUri, None)
+        self.assertEqual(ev.SecurityMode, None)
+        self.assertEqual(ev.RequestedLifetime, None)
 
     def test_eventgenerator_default(self):
         evgen = self.opc.get_event_generator()
@@ -291,6 +285,26 @@ class TestServer(unittest.TestCase, CommonTests, SubscriptionTests):
         self.assertEqual(ev.ServerId, None)
         self.assertEqual(ev.ClientAuditEntryId, None)
         self.assertEqual(ev.ClientUserId, None)
+        
+    def test_eventgenerator_MultiInheritedEvent(self):
+        evgen = self.opc.get_event_generator(ua.ObjectIds.AuditOpenSecureChannelEventType)
+        check_eventgenerator_SourceServer(self, evgen)
+
+        ev = evgen.event
+        self.assertIsNot(ev, None)  # we did not receive event
+        self.assertIsInstance(ev, BaseEvent)
+        self.assertIsInstance(ev, AuditEvent)
+        self.assertIsInstance(ev, AuditSecurityEvent)
+        self.assertIsInstance(ev, AuditChannelEvent)
+        self.assertIsInstance(ev, AuditOpenSecureChannelEvent)
+        self.assertEqual(ev.EventType, ua.NodeId(ua.ObjectIds.AuditOpenSecureChannelEventType))
+        self.assertEqual(ev.Severity, 1),
+        self.assertEqual(ev.ClientCertificate, None)
+        self.assertEqual(ev.ClientCertificateThumbprint, None)
+        self.assertEqual(ev.RequestType, None)
+        self.assertEqual(ev.SecurityPolicyUri, None)
+        self.assertEqual(ev.SecurityMode, None)
+        self.assertEqual(ev.RequestedLifetime, None)
 
     # For the custom events all posibilites are tested. For other custom types only one test case is done since they are using the same code
     def test_create_custom_data_type_ObjectId(self):
@@ -323,9 +337,9 @@ class TestServer(unittest.TestCase, CommonTests, SubscriptionTests):
         methods = node_type.get_methods()
         self.assertTrue(node_type.get_child("2:MyMethod") in methods)
 
-    #def test_create_custom_refrence_type_ObjectId(self):
-        #type = self.opc.create_custom_reference_type(2, 'MyEvent', ua.ObjectIds.Base, [('PropertyNum', ua.VariantType.Int32), ('PropertyString', ua.VariantType.String)])
-        #check_custom_type(self, type, ua.ObjectIds.BaseObjectType)
+    # def test_create_custom_refrence_type_ObjectId(self):
+        # type = self.opc.create_custom_reference_type(2, 'MyEvent', ua.ObjectIds.Base, [('PropertyNum', ua.VariantType.Int32), ('PropertyString', ua.VariantType.String)])
+        # check_custom_type(self, type, ua.ObjectIds.BaseObjectType)
 
     def test_create_custom_variable_type_ObjectId(self):
         type = self.opc.create_custom_variable_type(2, 'MyVariableType', ua.ObjectIds.BaseVariableType, [('PropertyNum', ua.VariantType.Int32), ('PropertyString', ua.VariantType.String)])
@@ -344,7 +358,7 @@ class TestServer(unittest.TestCase, CommonTests, SubscriptionTests):
 
         ev = opcua.common.events.get_event_obj_from_type_node(etype)
         check_custom_event(self, ev, etype)
-        self.assertEqual(ev.PropertyNum, None)
+        self.assertEqual(ev.PropertyNum, 0)
         self.assertEqual(ev.PropertyString, None)
 
     def test_eventgenerator_customEvent(self):
@@ -354,7 +368,7 @@ class TestServer(unittest.TestCase, CommonTests, SubscriptionTests):
         check_eventgenerator_CustomEvent(self, evgen, etype)
         check_eventgenerator_SourceServer(self, evgen)
 
-        self.assertEqual(evgen.event.PropertyNum, None)
+        self.assertEqual(evgen.event.PropertyNum, 0)
         self.assertEqual(evgen.event.PropertyString, None)
 
     def test_eventgenerator_double_customEvent(self):
@@ -367,12 +381,12 @@ class TestServer(unittest.TestCase, CommonTests, SubscriptionTests):
         check_eventgenerator_SourceServer(self, evgen)
 
         # Properties from MyEvent1
-        self.assertEqual(evgen.event.PropertyNum, None)
+        self.assertEqual(evgen.event.PropertyNum, 0)
         self.assertEqual(evgen.event.PropertyString, None)
 
          # Properties from MyEvent2
-        self.assertEqual(evgen.event.PropertyBool, None)
-        self.assertEqual(evgen.event.PropertyInt, None)
+        self.assertEqual(evgen.event.PropertyBool, False)
+        self.assertEqual(evgen.event.PropertyInt, 0)
 
     def test_eventgenerator_customEvent_MyObject(self):
         objects = self.opc.get_objects_node()
@@ -383,7 +397,7 @@ class TestServer(unittest.TestCase, CommonTests, SubscriptionTests):
         check_eventgenerator_CustomEvent(self, evgen, etype)
         check_event_generator_object(self, evgen, o)
 
-        self.assertEqual(evgen.event.PropertyNum, None)
+        self.assertEqual(evgen.event.PropertyNum, 0)
         self.assertEqual(evgen.event.PropertyString, None)
 
     def test_context_manager(self):
@@ -396,7 +410,7 @@ class TestServer(unittest.TestCase, CommonTests, SubscriptionTests):
         # create server and replace instance methods with dummy methods
         server = Server()
         server.start = increment_state.__get__(server)
-        server.stop  = increment_state.__get__(server)
+        server.stop = increment_state.__get__(server)
 
         assert state[0] == 0
         with server:
@@ -405,6 +419,59 @@ class TestServer(unittest.TestCase, CommonTests, SubscriptionTests):
         # test if server stopped
         self.assertEqual(state[0], 2)
 
+    def test_get_node_by_ns(self):
+
+        def get_ns_of_nodes(nodes):
+            ns_list = set()
+            for node in nodes:
+                ns_list.add(node.nodeid.NamespaceIndex)
+            return ns_list
+
+        # incase other testss created nodes  in unregistered namespace
+        _idx_d = self.opc.register_namespace('dummy1')
+        _idx_d = self.opc.register_namespace('dummy2')
+        _idx_d = self.opc.register_namespace('dummy3')
+
+        # create the test namespaces and vars
+        idx_a = self.opc.register_namespace('a')
+        idx_b = self.opc.register_namespace('b')
+        idx_c = self.opc.register_namespace('c')
+        o = self.opc.get_objects_node()
+        _myvar2 = o.add_variable(idx_a, "MyBoolVar2", True)
+        _myvar3 = o.add_variable(idx_b, "MyBoolVar3", True)
+        _myvar4 = o.add_variable(idx_c, "MyBoolVar4", True)
+
+        # the tests
+        nodes = ua_utils.get_nodes_of_namespace(self.opc, namespaces=[idx_a, idx_b, idx_c])
+        self.assertEqual(len(nodes), 3)
+        self.assertEqual(get_ns_of_nodes(nodes), set([idx_a, idx_b, idx_c]))
+
+        nodes = ua_utils.get_nodes_of_namespace(self.opc, namespaces=[idx_a])
+        self.assertEqual(len(nodes), 1)
+        self.assertEqual(get_ns_of_nodes(nodes), set([idx_a]))
+
+        nodes = ua_utils.get_nodes_of_namespace(self.opc, namespaces=[idx_b])
+        self.assertEqual(len(nodes), 1)
+        self.assertEqual(get_ns_of_nodes(nodes), set([idx_b]))
+
+        nodes = ua_utils.get_nodes_of_namespace(self.opc, namespaces=['a'])
+        self.assertEqual(len(nodes), 1)
+        self.assertEqual(get_ns_of_nodes(nodes), set([idx_a]))
+
+        nodes = ua_utils.get_nodes_of_namespace(self.opc, namespaces=['a', 'c'])
+        self.assertEqual(len(nodes), 2)
+        self.assertEqual(get_ns_of_nodes(nodes), set([idx_a, idx_c]))
+
+        nodes = ua_utils.get_nodes_of_namespace(self.opc, namespaces='b')
+        self.assertEqual(len(nodes), 1)
+        self.assertEqual(get_ns_of_nodes(nodes), set([idx_b]))
+
+        nodes = ua_utils.get_nodes_of_namespace(self.opc, namespaces=idx_b)
+        self.assertEqual(len(nodes), 1)
+        self.assertEqual(get_ns_of_nodes(nodes), set([idx_b]))
+
+        self.assertRaises(ValueError, ua_utils.get_nodes_of_namespace, self.opc, namespaces='non_existing_ns')
+
 def check_eventgenerator_SourceServer(test, evgen):
     server = test.opc.get_server_node()
     test.assertEqual(evgen.event.SourceName, server.get_browse_name().Name)
@@ -475,13 +542,13 @@ def check_custom_type(test, type, base_type):
 
 class TestServerCaching(unittest.TestCase):
     def runTest(self):
+        return # FIXME broken
         tmpfile = NamedTemporaryFile()
         path = tmpfile.name
         tmpfile.close()
 
         # create cache file
-        server = Server()
-        server.iserver.dump_address_space(path)
+        server = Server(shelffile=path)
 
         # modify cache content
         id = ua.NodeId(ua.ObjectIds.Server_ServerStatus_SecondsTillShutdown)
@@ -490,7 +557,7 @@ class TestServerCaching(unittest.TestCase):
         s.close()
 
         # ensure that we are actually loading from the cache
-        server = Server(cacheFile=path)
+        server = Server(shelffile=path)
         self.assertEqual(server.get_node(id).get_value(), 123)
 
         os.remove(path)
