@@ -5,6 +5,7 @@ import io
 from datetime import datetime
 import unittest
 from collections import namedtuple
+import uuid
 
 from opcua import ua
 from opcua.ua import extensionobject_from_binary
@@ -13,8 +14,9 @@ from opcua.ua.uatypes import flatten, get_shape, reshape
 from opcua.server.internal_subscription import WhereClauseEvaluator
 from opcua.common.event_objects import BaseEvent
 from opcua.common.ua_utils import string_to_variant, variant_to_string, string_to_val, val_to_string
-from opcua.common.xmlparser import XMLParser
+from opcua.common.xmlimporter import XmlImporter
 from opcua.ua.uatypes import _MaskEnum
+from opcua.common.structures_generator import StructGenerator
 
 
 class TestUnit(unittest.TestCase):
@@ -23,6 +25,141 @@ class TestUnit(unittest.TestCase):
     Simple unit test that do not need to setup a server or a client
     '''
 
+    def test_variant_array_none(self):
+        v = ua.Variant(None, varianttype=ua.VariantType.Int32, is_array=True)
+        data = v.to_binary()
+        v2 = ua.Variant.from_binary(ua.utils.Buffer(data))
+        self.assertEqual(v, v2)
+        self.assertTrue(v2.is_array)
+
+        v = ua.Variant(None, varianttype=ua.VariantType.Null, is_array=True)
+        data = v.to_binary()
+        v2 = ua.Variant.from_binary(ua.utils.Buffer(data))
+        self.assertEqual(v, v2)
+        self.assertTrue(v2.is_array)
+
+    def test_structs_save_and_import(self):
+        xmlpath = "tests/example.bsd"
+        c = StructGenerator()
+        c.make_model_from_file(xmlpath)
+        struct_dict = c.save_and_import("structures.py")
+        for k, v in struct_dict.items():
+            a = v()
+            self.assertEqual(k, a.__class__.__name__)
+
+    def test_custom_structs(self):
+        xmlpath = "tests/example.bsd"
+        c = StructGenerator()
+        c.make_model_from_file(xmlpath)
+        c.save_to_file("structures.py")
+        import structures as s
+
+        # test with default values
+        v = s.ScalarValueDataType()
+        data = v.to_binary()
+        v2 = s.ScalarValueDataType.from_binary(ua.utils.Buffer(data))
+
+
+        # set some values
+        v = s.ScalarValueDataType()
+        v.SbyteValue = 1
+        v.ByteValue = 2
+        v.Int16Value = 3
+        v.UInt16Value = 4
+        v.Int32Value = 5
+        v.UInt32Value = 6
+        v.Int64Value = 7
+        v.UInt64Value = 8
+        v.FloatValue = 9.0
+        v.DoubleValue = 10.0
+        v.StringValue = "elleven"
+        v.DateTimeValue = datetime.utcnow()
+        #self.GuidValue = uuid.uudib"14"
+        v.ByteStringValue = b"fifteen"
+        v.XmlElementValue = ua.XmlElement("<toto>titi</toto>")
+        v.NodeIdValue = ua.NodeId.from_string("ns=4;i=9999")
+        #self.ExpandedNodeIdValue =
+        #self.QualifiedNameValue =
+        #self.LocalizedTextValue =
+        #self.StatusCodeValue =
+        #self.VariantValue =
+        #self.EnumerationValue =
+        #self.StructureValue =
+        #self.Number =
+        #self.Integer =
+        #self.UInteger =
+
+
+        data = v.to_binary()
+        v2 = s.ScalarValueDataType.from_binary(ua.utils.Buffer(data))
+        self.assertEqual(v.NodeIdValue, v2.NodeIdValue)
+
+    def test_custom_structs_array(self):
+        xmlpath = "tests/example.bsd"
+        c = StructGenerator()
+        c.make_model_from_file(xmlpath)
+        c.save_to_file("structures.py")
+        import structures as s
+
+        # test with default values
+        v = s.ArrayValueDataType()
+        data = v.to_binary()
+        v2 = s.ArrayValueDataType.from_binary(ua.utils.Buffer(data))
+
+
+        # set some values
+        v = s.ArrayValueDataType()
+        v.SbyteValue = [1]
+        v.ByteValue = [2]
+        v.Int16Value = [3]
+        v.UInt16Value = [4]
+        v.Int32Value = [5]
+        v.UInt32Value = [6]
+        v.Int64Value = [7]
+        v.UInt64Value = [8]
+        v.FloatValue = [9.0]
+        v.DoubleValue = [10.0]
+        v.StringValue = ["elleven"]
+        v.DateTimeValue = [datetime.utcnow()]
+        #self.GuidValue = uuid.uudib"14"
+        v.ByteStringValue = [b"fifteen", b"sixteen"]
+        v.XmlElementValue = [ua.XmlElement("<toto>titi</toto>")]
+        v.NodeIdValue = [ua.NodeId.from_string("ns=4;i=9999"), ua.NodeId.from_string("i=6")]
+        #self.ExpandedNodeIdValue =
+        #self.QualifiedNameValue =
+        #self.LocalizedTextValue =
+        #self.StatusCodeValue =
+        #self.VariantValue =
+        #self.EnumerationValue =
+        #self.StructureValue =
+        #self.Number =
+        #self.Integer =
+        #self.UInteger =
+
+
+        data = v.to_binary()
+        v2 = s.ArrayValueDataType.from_binary(ua.utils.Buffer(data))
+        self.assertEqual(v.NodeIdValue, v2.NodeIdValue)
+        print(v2.NodeIdValue)
+
+
+    def test_nodeid_ordering(self):
+        a = ua.NodeId(2000, 1)
+        b = ua.NodeId(3000, 1)
+        c = ua.NodeId(20, 0)
+        d = ua.NodeId("tititu", 1)
+        e = ua.NodeId("aaaaa", 1)
+        f = ua.NodeId("aaaaa", 2)
+        g = ua.NodeId(uuid.uuid4(), 1)
+        h = ua.TwoByteNodeId(2001)
+        i = ua.NodeId(b"lkjkl", 1, ua.NodeIdType.ByteString)
+        j = ua.NodeId(b"aaa", 5, ua.NodeIdType.ByteString)
+
+        mylist = [a, b, c, d, e, f, g, h, i, j]
+        mylist.sort()
+        expected = [c, h, a, b, e, d, f, g, i, j]
+        self.assertEqual(mylist, expected)
+
     def test_string_to_variant_int(self):
         s_arr_uint = "[1, 2, 3, 4]"
         arr_uint = [1, 2, 3, 4]
@@ -46,7 +183,7 @@ class TestUnit(unittest.TestCase):
         s_datetime = "2014-05-3"
         self.assertEqual(val_to_string(arr_string), s_arr_datetime)
         self.assertEqual(string_to_val(s_arr_datetime, ua.VariantType.String), arr_string)
-        self.assertEqual(string_to_val(s_arr_datetime, ua.VariantType.DateTime), arr_datetime )
+        self.assertEqual(string_to_val(s_arr_datetime, ua.VariantType.DateTime), arr_datetime)
 
     def test_string_to_variant_nodeid(self):
         s_arr_nodeid = "[ns=2;i=56, i=45]"
@@ -69,19 +206,30 @@ class TestUnit(unittest.TestCase):
 
     def test_string_to_variant_localized_text(self):
         string = "_This is my string"
-        #string = "_This is my nøåæ"FIXME: does not work with python2 ?!?!
+        # string = "_This is my nøåæ"FIXME: does not work with python2 ?!?!
         obj = ua.LocalizedText(string)
         self.assertEqual(string_to_val(string, ua.VariantType.LocalizedText), obj)
         self.assertEqual(val_to_string(obj), string)
 
+    def test_string_to_val_xml_element(self):
+        string = "<p> titi toto </p>"
+        obj = ua.XmlElement(string)
+
+        self.assertEqual(string_to_val(string, ua.VariantType.XmlElement), obj)
+        self.assertEqual(val_to_string(obj), string)
+
+        b = obj.to_binary()
+        obj2 = obj.from_binary(ua.utils.Buffer(b))
+        self.assertEqual(obj, obj2)
+
     def test_variant_dimensions(self):
-        l = [[[1.0, 1.0, 1.0, 1.0], [2.0, 2.0, 2.0, 2.0], [3.0, 3.0, 3.0, 3.0]],[[5.0, 5.0, 5.0, 5.0], [7.0, 8.0, 9.0, 01.0], [1.0, 1.0, 1.0, 1.0]]]
+        l = [[[1.0, 1.0, 1.0, 1.0], [2.0, 2.0, 2.0, 2.0], [3.0, 3.0, 3.0, 3.0]], [[5.0, 5.0, 5.0, 5.0], [7.0, 8.0, 9.0, 01.0], [1.0, 1.0, 1.0, 1.0]]]
         v = ua.Variant(l)
         self.assertEqual(v.Dimensions, [2, 3, 4])
         v2 = ua.Variant.from_binary(ua.utils.Buffer(v.to_binary()))
         self.assertEqual(v, v2)
         self.assertEqual(v.Dimensions, v2.Dimensions)
-        
+
         # very special case
         l = [[[], [], []], [[], [], []]]
         v = ua.Variant(l, ua.VariantType.UInt32)
@@ -91,13 +239,13 @@ class TestUnit(unittest.TestCase):
         self.assertEqual(v, v2)
 
     def test_flatten(self):
-        l = [[[1.0, 1.0, 1.0, 1.0], [1.0, 1.0, 1.0, 1.0], [1.0, 1.0, 1.0, 1.0]],[[1.0, 1.0, 1.0, 1.0], [1.0, 1.0, 1.0, 1.0], [1.0, 1.0, 1.0, 1.0]]]
+        l = [[[1.0, 1.0, 1.0, 1.0], [1.0, 1.0, 1.0, 1.0], [1.0, 1.0, 1.0, 1.0]], [[1.0, 1.0, 1.0, 1.0], [1.0, 1.0, 1.0, 1.0], [1.0, 1.0, 1.0, 1.0]]]
         l2 = flatten(l)
         dims = get_shape(l)
         self.assertEqual(dims, [2, 3, 4])
         self.assertNotEqual(l, l2)
 
-        l3 = reshape(l2, (2,3,4))
+        l3 = reshape(l2, (2, 3, 4))
         self.assertEqual(l, l3)
 
 
@@ -128,8 +276,10 @@ class TestUnit(unittest.TestCase):
         self.assertEqual(v, v2)
 
     def test_guid(self):
-        g = ua.Guid()
-        sc = ua.StatusCode()
+        v = ua.Variant(uuid.uuid4(), ua.VariantType.Guid)
+        v2 = ua.Variant.from_binary(ua.utils.Buffer(v.to_binary()))
+        self.assertEqual(v.VariantType, v2.VariantType)
+        self.assertEqual(v, v2)
 
     def test_nodeid(self):
         nid = ua.NodeId()
@@ -150,8 +300,8 @@ class TestUnit(unittest.TestCase):
         s = ua.StringNodeId(53, 0)  # should we raise an exception???
         s1 = ua.StringNodeId("53", 0)
         bs = ua.ByteStringNodeId(b"53", 0)
-        gid = ua.Guid()
-        g = ua.ByteStringNodeId(gid, 0)
+        gid = uuid.uuid4()
+        g = ua.ByteStringNodeId(str(gid), 0)
         guid = ua.GuidNodeId(gid)
         self.assertEqual(tb, fb)
         self.assertEqual(tb, n)
@@ -178,7 +328,7 @@ class TestUnit(unittest.TestCase):
         self.assertNotEqual(nid, ua.NodeId.from_string("i=5; ns=10"))
         # not sure the next one is correct...
         self.assertEqual(nid, ua.NodeId.from_string("i=45; ns=10; srv=serverid"))
-        
+
         nid1 = ua.NodeId("myid.mynodeid", 7)
         self.assertEqual(nid1, ua.NodeId.from_string("ns=7; s=myid.mynodeid"))
         with self.assertRaises(ua.UaError):
@@ -298,7 +448,7 @@ class TestUnit(unittest.TestCase):
     def test_strrepr_nodeid(self):
         nid = ua.NodeId.from_string('ns=2;s=PLC1.Manufacturer;')
         self.assertEqual(nid.to_string(), 'ns=2;s=PLC1.Manufacturer')
-        #self.assertEqual(repr(nid), 'ns=2;s=PLC1.Manufacturer;')
+        # self.assertEqual(repr(nid), 'ns=2;s=PLC1.Manufacturer;')
 
     def test_qualified_name(self):
         qn = ua.QualifiedName('qname', 2)
@@ -346,10 +496,10 @@ class TestUnit(unittest.TestCase):
         self.assertEqual(v.VariantType, v2.VariantType)
 
     def test_variant_array_dim(self):
-        v = ua.Variant([1, 2, 3, 4, 5, 6], dimensions = [2, 3])
+        v = ua.Variant([1, 2, 3, 4, 5, 6], dimensions=[2, 3])
         self.assertEqual(v.Value[1], 2)
         v2 = ua.Variant.from_binary(ua.utils.Buffer(v.to_binary()))
-        self.assertEqual(reshape(v.Value, (2,3)), v2.Value)
+        self.assertEqual(reshape(v.Value, (2, 3)), v2.Value)
         self.assertEqual(v.VariantType, v2.VariantType)
         self.assertEqual(v.Dimensions, v2.Dimensions)
         self.assertEqual(v2.Dimensions, [2, 3])
@@ -440,52 +590,6 @@ class TestUnit(unittest.TestCase):
 
         self.assertTrue(wce.eval(ev))
 
-    def test_xmlparser_get_node_id(self):
-        server = None
-        xmlpath = 'tests/custom_nodes.xml'
-        parser = XMLParser(xmlpath, server)
-
-        res1 = parser._get_node_id('i=1001')
-        self.assertEqual(res1, 'i=1001')
-
-        res2 = parser._get_node_id('ns=1;i=1001')
-        self.assertEqual(res2, 'ns=1;i=1001')
-
-        parser.namespaces = {1: [3, 'http://someuri.com']}
-        res3 = parser._get_node_id('ns=1;i=1001')
-        self.assertEqual(res3, 'ns=3;i=1001')
-
-        parser.namespaces = {1: [3, 'http://someuri.com']}
-        res4 = parser._get_node_id('ns=2;i=1001')
-        self.assertEqual(res4, 'ns=2;i=1001')
-
-    def test_xmlparser_sort_nodes_by_parentid(self):
-        NodeMock = namedtuple('NodeMock', 'nodeid parent')
-
-        server = None
-        # We actually dont need this. Thus we just pass the available file
-        xml_path = 'tests/custom_nodes.xml'
-
-        unordered_nodes = [
-            NodeMock('ns=1;i=1001', None),
-            NodeMock('ns=1;i=1002', 'ns=1;i=1003'),
-            NodeMock('ns=1;i=1003', 'ns=1;i=1001'),
-            NodeMock('ns=1;i=1004', 'ns=1;i=1002')
-        ]
-
-        ordered_nodes = [
-            unordered_nodes[0],
-            unordered_nodes[2],
-            unordered_nodes[1],
-            unordered_nodes[3],
-        ]
-        namespaces = {'1': (1, 'http://someuri.com')}
-
-        parser = XMLParser(xml_path, server)
-        parser.namespaces = namespaces
-        res = parser._sort_nodes_by_parentid(unordered_nodes)
-        self.assertEqual(res, ordered_nodes)
-
 
 class TestMaskEnum(unittest.TestCase):
     class MyEnum(_MaskEnum):
